@@ -1,6 +1,6 @@
-#!/usr/local/bin/python3.8
+#!/usr/local/bin/python3.9
 ###############################################################################
-# $Id: rel.py e4fe7cc06270e5f38dfe78e6785a6bcca4e39e29 2021-04-01 21:02:04 +0300 Idan Miara $
+# $Id: rel.py ae12632821c260963247432aff038b3b7244361d 2021-04-27 23:42:00 +0300 Idan Miara $
 #
 # Project:  GDAL Python samples
 # Purpose:  Script to produce a shaded relief image from elevation data
@@ -92,7 +92,7 @@ def main(argv):
     infile = None
     outfile = None
     iBand = 1	    # The first band will be converted by default
-    frmt = 'GTiff'
+    driver_name = 'GTiff'
     typ = gdal.GDT_Byte
 
     lsrcaz = None
@@ -177,7 +177,7 @@ def main(argv):
         print('Input image is too small to process, minimum size is 3x3')
         return 3
 
-    out_driver = gdal.GetDriverByName(frmt)
+    out_driver = gdal.GetDriverByName(driver_name)
     outdataset = out_driver.Create(outfile, indataset.RasterXSize, indataset.RasterYSize, indataset.RasterCount, typ)
     outband = outdataset.GetRasterBand(1)
 
