@@ -1,7 +1,7 @@
-#!/usr/local/bin/python3.8
+#!/usr/local/bin/python3.9
 # -*- coding: utf-8 -*-
 # *****************************************************************************
-#  $Id: validate_cloud_optimized_geotiff.py d464f5b35ba2c5ad85fe60b723509122909a7127 2020-05-15 20:32:31 +0200 Even Rouault $
+#  $Id: validate_cloud_optimized_geotiff.py e4fe7cc06270e5f38dfe78e6785a6bcca4e39e29 2021-04-01 21:02:04 +0300 Idan Miara $
 #
 #  Project:  GDAL
 #  Purpose:  Validate Cloud Optimized GeoTIFF file structure
@@ -337,26 +337,26 @@ def validate(ds, check_tiled=True, full_check=False):
     return warnings, errors, details
 
 
-def main():
+def main(argv):
     """Return 0 in case of success, 1 for failure."""
 
     i = 1
     filename = None
     quiet = False
     full_check = None
-    while i < len(sys.argv):
-        if sys.argv[i] == '-q':
+    while i < len(argv):
+        if argv[i] == '-q':
             quiet = True
-        elif sys.argv[i] == '--full-check=yes':
+        elif argv[i] == '--full-check=yes':
             full_check = True
-        elif sys.argv[i] == '--full-check=no':
+        elif argv[i] == '--full-check=no':
             full_check = False
-        elif sys.argv[i] == '--full-check=auto':
+        elif argv[i] == '--full-check=auto':
             full_check = None
-        elif sys.argv[i][0] == '-':
+        elif argv[i][0] == '-':
             return Usage()
         elif filename is None:
-            filename = sys.argv[i]
+            filename = argv[i]
         else:
             return Usage()
 
@@ -404,4 +404,4 @@ def main():
 
 
 if __name__ == '__main__':
-    sys.exit(main())
+    sys.exit(main(sys.argv))
