@@ -1,7 +1,7 @@
-#!/usr/local/bin/python3.8
+#!/usr/local/bin/python3.9
 # -*- coding: utf-8 -*-
 ###############################################################################
-# $Id: validate_gpkg.py 742ea8f115ab2969fdc0251eae2a3bfaaadef0e4 2020-06-27 18:29:12 +0200 Even Rouault $
+# $Id: validate_gpkg.py d83893162b4215f023ced16148754b836d44e2d1 2022-01-31 00:12:08 +0100 Even Rouault $
 #
 # Project:  GDAL/OGR
 # Purpose:  Test compliance of GeoPackage database w.r.t GeoPackage spec
@@ -31,6 +31,7 @@
 
 import datetime
 import os
+import re
 import sqlite3
 import struct
 import sys
@@ -39,6 +40,7 @@ import sys
 # If not available, those tests will be skipped
 try:
     from osgeo import gdal
+    from osgeo import ogr
     has_gdal = True
 except ImportError:
     has_gdal = False
@@ -65,18 +67,29 @@ class GPKGCheckException(Exception):
 
 class GPKGChecker(object):
 
+    BASE_GEOM_TYPES = ('GEOMETRY', 'POINT', 'LINESTRING', 'POLYGON',
+                       'MULTIPOINT', 'MULTILINESTRING', 'MULTIPOLYGON',
+                       'GEOMETRYCOLLECTION')
     EXT_GEOM_TYPES = ('CIRCULARSTRING', 'COMPOUNDCURVE', 'CURVEPOLYGON',
                       'MULTICURVE', 'MULTISURFACE', 'CURVE', 'SURFACE')
 
-    def __init__(self, filename, abort_at_first_error=True, verbose=False):
+    def __init__(self, filename, abort_at_first_error=True,
+                 extra_checks=False,
+                 log_msg=False,
+                 warning_msg=True,
+                 warning_as_error=False):
         self.filename = filename
         self.extended_pragma_info = False
         self.abort_at_first_error = abort_at_first_error
-        self.verbose = verbose
+        self.extra_checks = extra_checks
+        self.log_msg = log_msg
+        self.warning_msg = warning_msg
+        self.warning_as_error = warning_as_error
         self.errors = []
+        self.warnings = []
 
     def _log(self, msg):
-        if self.verbose:
+        if self.log_msg:
             print(msg)
 
     def _assert(self, cond, req, msg):
@@ -90,6 +103,14 @@ class GPKGChecker(object):
                     raise GPKGCheckException(msg)
         return cond
 
+    def _warn(self, msg):
+        if self.warning_as_error:
+            self._assert(False, None, msg)
+        else:
+            self.warnings += [msg]
+            if self.warning_msg:
+                print(msg)
+
     def _check_structure(self, columns, expected_columns, req, table_name):
         self._assert(len(columns) == len(expected_columns), req,
                      'Table %s has %d columns, whereas %d are expected' %
@@ -141,29 +162,41 @@ class GPKGChecker(object):
         c.execute("PRAGMA table_info(gpkg_spatial_ref_sys)")
         columns = c.fetchall()
         has_definition_12_063 = False
+        has_epoch = False
         for (_, name, _, _, _, _) in columns:
             if name == 'definition_12_063':
                 has_definition_12_063 = True
+            if name == 'epoch':
+                has_epoch = True
 
         c.execute("SELECT 1 FROM sqlite_master WHERE name = 'gpkg_extensions'")
         row = None
         if c.fetchone() is not None:
-            c.execute("SELECT scope FROM gpkg_extensions WHERE "
-                      "extension_name = 'gpkg_crs_wkt'")
+            c.execute("SELECT scope, extension_name FROM gpkg_extensions WHERE "
+                      "extension_name IN ('gpkg_crs_wkt', 'gpkg_crs_wkt_1_1')")
             row = c.fetchone()
         if row:
-            scope, = row
+            scope, extension_name = row
             self._assert(scope == 'read-write', 145,
-                         'scope of gpkg_crs_wkt extension should be read-write')
+                         f'scope of {extension_name} extension should be read-write')
             self._assert(
                 has_definition_12_063, 145,
                 "gpkg_spatial_ref_sys should have a definition_12_063 column, "
-                "as gpkg_crs_wkt extension is declared")
+                f"as {extension_name} extension is declared")
+            if has_epoch:
+                self._assert(
+                    extension_name == 'gpkg_crs_wkt_1_1', 145,
+                    "gpkg_extensions should declare gpkg_crs_wkt_1_1 extension "
+                    "as gpkg_spatial_ref_sys has a epoch column")
         else:
             self._assert(
                 not has_definition_12_063, 145,
                 "gpkg_extensions should declare gpkg_crs_wkt extension "
                 "as gpkg_spatial_ref_sys has a definition_12_063 column")
+            self._assert(
+                not has_epoch, 145,
+                "gpkg_extensions should declare gpkg_crs_wkt_1_1 extension "
+                "as gpkg_spatial_ref_sys has a epoch column")
 
         if has_definition_12_063:
             expected_columns = [
@@ -175,6 +208,8 @@ class GPKGChecker(object):
                 (5, 'description', 'TEXT', 0, None, 0),
                 (6, 'definition_12_063', 'TEXT', 1, None, 0)
             ]
+            if has_epoch:
+                expected_columns += [(7, 'epoch', 'DOUBLE', 0, None, 0)]
         else:
             expected_columns = [
                 (0, 'srs_name', 'TEXT', 1, None, 0),
@@ -280,17 +315,11 @@ class GPKGChecker(object):
         ]
         self._check_structure(columns, expected_columns, 13, 'gpkg_contents')
 
-        c.execute("SELECT 1 FROM gpkg_contents "
-                  "WHERE data_type IN ('features', 'tiles')")
-        self._assert(c.fetchone() is not None, 17,
-                     'gpkg_contents should at least have one table with '
-                     'data_type = features and/or tiles')
-
         c.execute("SELECT table_name, data_type FROM gpkg_contents "
                   "WHERE data_type NOT IN "
                   "('features', 'tiles', 'attributes', '2d-gridded-coverage')")
         ret = c.fetchall()
-        self._assert(len(ret) == 0, 17,
+        self._assert(len(ret) == 0, 17,  # no longer required actually...
                      'Unexpected data types in gpkg_contents: %s' % str(ret))
 
         c.execute('SELECT table_name, last_change, srs_id FROM gpkg_contents')
@@ -319,6 +348,126 @@ class GPKGChecker(object):
                               "which isn't found in gpkg_spatial_ref_sys") %
                              (table_name, srs_id))
 
+    def _check_user_table_content(self, c, table_name, cols):
+
+        if not self.extra_checks:
+            self._log('Skipping checks on actual table content')
+            return
+
+        self._log('Checking actual table content')
+
+        warned_col = [False for col in cols]
+        c.execute("SELECT %s FROM %s" % (
+            ','.join(("%s,typeof(%s),length(%s)" %
+                      (_esc_id(col[1]), _esc_id(col[1]), _esc_id(col[1]))) for col in cols),
+            _esc_id(table_name)))
+        geom_types = GPKGChecker.BASE_GEOM_TYPES + GPKGChecker.EXT_GEOM_TYPES
+        date_pattern_str = '[0-9]{4}-[0-1][0-9]-[0-3][0-9]'
+        date_pattern = re.compile(date_pattern_str)
+        datetime_pattern = re.compile(
+            date_pattern_str + 'T[0-2][0-9]:[0-5][0-9]:[0-5][0-9].[0-9]{3}Z')
+        for row in c.fetchall():
+            for i in range(len(cols)):
+                if warned_col[i]:
+                    continue
+
+                val = row[3*i+0]
+                got_col_type = row[3*i+1].upper()
+                length_value = row[3*i+2]
+                if val is None:
+                    continue
+                expected_col_type = cols[i][2]
+                col_name = cols[i][1]
+
+                if expected_col_type == 'BOOLEAN' and got_col_type == 'INTEGER':
+                    if val not in (0, 1):
+                        warned_col[i] = True
+                        self._warn('In column %s, value %d found whereas 0 or 1 was expected' %
+                                   (col_name, val))
+                        continue
+                    continue
+
+                elif expected_col_type == 'TINYINT' and got_col_type == 'INTEGER':
+                    if val < -128 or val > 127:
+                        warned_col[i] = True
+                        self._warn(
+                            'In column %s, value %d found whereas it was expected to be in [-128,127]' % (col_name, val))
+                        continue
+                    continue
+
+                elif expected_col_type == 'SMALLINT' and got_col_type == 'INTEGER':
+                    if val < -32768 or val > 32767:
+                        warned_col[i] = True
+                        self._warn(
+                            'In column %s, value %d found whereas it was expected to be in [-32768,32767]' % (col_name, val))
+                        continue
+                    continue
+
+                elif expected_col_type == 'MEDIUMINT' and got_col_type == 'INTEGER':
+                    if val < -2147483648 or val > 2147483647:
+                        warned_col[i] = True
+                        self._warn(
+                            'In column %s, value %d found whereas it was expected to be in [-2147483648,2147483647]' % (col_name, val))
+                        continue
+                    continue
+
+                elif expected_col_type in ('INT', 'INTEGER') and got_col_type == 'INTEGER':
+                    continue
+
+                elif expected_col_type in ('FLOAT', 'DOUBLE', 'REAL') and got_col_type == 'REAL':
+                    continue
+
+                elif expected_col_type.startswith('TEXT('):
+                    if got_col_type != 'TEXT':
+                        warned_col[i] = True
+                        self._warn('In column %s, content of type %s found whereas %s was expected' % (
+                            col_name, got_col_type, expected_col_type))
+                        continue
+
+                    expected_max_length = int(expected_col_type[len('TEXT('): -1])
+                    if length_value > expected_max_length:
+                        warned_col[i] = True
+                        self._warn('In column %s, string of length %d found whereas %d max was expected' % (
+                            col_name, length_value, expected_max_length))
+                        continue
+
+                elif expected_col_type.startswith('BLOB('):
+                    if got_col_type != 'BLOB':
+                        warned_col[i] = True
+                        self._warn('In column %s, content of type %s found whereas %s was expected' % (
+                            col_name, got_col_type, expected_col_type))
+                        continue
+
+                    expected_max_length = int(expected_col_type[len('BLOB('): -1])
+                    if length_value > expected_max_length:
+                        warned_col[i] = True
+                        self._warn('In column %s, blob of length %d found whereas %d max was expected' % (
+                            col_name, length_value, expected_max_length))
+                        continue
+
+                elif got_col_type == 'BLOB' and expected_col_type in geom_types:
+                    continue
+
+                elif expected_col_type == 'DATE' and got_col_type == 'TEXT':
+                    if date_pattern.match(val) is None:
+                        warned_col[i] = True
+                        self._warn('In column %s, text %s found which is not a valid DATE' %
+                                   (col_name, val))
+                        continue
+
+                elif expected_col_type == 'DATETIME' and got_col_type == 'TEXT':
+                    if datetime_pattern.match(val) is None:
+                        warned_col[i] = True
+                        self._warn(
+                            'In column %s, text %s found which is not a valid DATETIME' % (col_name, val))
+                        continue
+
+                elif expected_col_type != got_col_type:
+                    warned_col[i] = True
+                    self._warn('In column %s, content of type %s found whereas %s was expected' % (
+                        col_name, got_col_type, expected_col_type))
+                    continue
+
     def _check_vector_user_table(self, c, table_name):
 
         self._log('Checking vector user table ' + table_name)
@@ -337,9 +486,6 @@ class GPKGChecker(object):
         srs_id = rows_gpkg_geometry_columns[0][4]
 
         c.execute('PRAGMA table_info(%s)' % _esc_id(table_name))
-        base_geom_types = ('GEOMETRY', 'POINT', 'LINESTRING', 'POLYGON',
-                           'MULTIPOINT', 'MULTILINESTRING', 'MULTIPOLYGON',
-                           'GEOMETRYCOLLECTION')
         cols = c.fetchall()
         found_geom = False
         count_pkid = 0
@@ -347,7 +493,7 @@ class GPKGChecker(object):
             if name.lower() == geom_column_name.lower():
                 found_geom = True
                 self._assert(
-                    typ in base_geom_types or
+                    typ in GPKGChecker.BASE_GEOM_TYPES or
                     typ in GPKGChecker.EXT_GEOM_TYPES,
                     25, ('invalid type (%s) for geometry ' +
                          'column of table %s') % (typ, table_name))
@@ -370,8 +516,24 @@ class GPKGChecker(object):
         self._assert(found_geom, 24,
                      'table %s has no %s column' %
                      (table_name, geom_column_name))
-        self._assert(count_pkid == 1, 29,
-                     'table %s has no INTEGER PRIMARY KEY' % table_name)
+
+        c.execute("SELECT 1 FROM sqlite_master WHERE "
+                  "type = 'table' AND name = ?", (table_name,))
+        if c.fetchone():
+            self._assert(count_pkid == 1, 29,
+                         'table %s has no INTEGER PRIMARY KEY' % table_name)
+
+            self._check_user_table_content(c, table_name, cols)
+
+        else:
+            self._assert(len(cols) > 0 and cols[0][2] == 'INTEGER',
+                         150, 'view %s has no INTEGER first column' % table_name)
+
+            c.execute("SELECT COUNT(*) - COUNT(DISTINCT %s) FROM %s" %
+                      (_esc_id(cols[0][1]), _esc_id(table_name)))
+            self._assert(c.fetchone()[0] == 0, 150,
+                         'First column of view %s should contain '
+                         'unique values' % table_name)
 
         self._assert(z in (0, 1, 2), 27, ("z value of %s is %d. " +
                                           "Expected 0, 1 or 2") % (table_name, z))
@@ -389,13 +551,14 @@ class GPKGChecker(object):
                          "gpkg_geom_%s extension should be declared for "
                          "table %s" % (geometry_type_name, table_name))
 
-        wkb_geometries = base_geom_types + GPKGChecker.EXT_GEOM_TYPES
+        wkb_geometries = GPKGChecker.BASE_GEOM_TYPES + GPKGChecker.EXT_GEOM_TYPES
         c.execute("SELECT %s FROM %s " %
                   (_esc_id(geom_column_name), _esc_id(table_name)))
         found_geom_types = set()
         for (blob,) in c.fetchall():
             if blob is None:
                 continue
+
             self._assert(len(blob) >= 8, 19, 'Invalid geometry')
             max_size_needed = min(len(blob), 8 + 4 * 2 * 8 + 5)
             blob_ar = struct.unpack('B' * max_size_needed,
@@ -404,6 +567,7 @@ class GPKGChecker(object):
             self._assert(blob_ar[1] == ord('P'), 19, 'Invalid geometry')
             self._assert(blob_ar[2] == 0, 19, 'Invalid geometry')
             flags = blob_ar[3]
+            empty_flag = ((flags >> 3) & 1) == 1
             big_endian = (flags & 1) == 0
             env_ind = (flags >> 1) & 7
             self._assert(((flags >> 5) & 1) == 0, 19,
@@ -411,14 +575,16 @@ class GPKGChecker(object):
                          'allowed')
             self._assert(env_ind <= 4, 19,
                          'Invalid geometry: invalid envelope indicator code')
-            if big_endian:
-                geom_srs_id = struct.unpack('>I' * 1, blob[4:8])[0]
-            else:
-                geom_srs_id = struct.unpack('<I' * 1, blob[4:8])[0]
+            endian_prefix = '>' if big_endian else '<'
+            geom_srs_id = struct.unpack((endian_prefix + 'I') * 1, blob[4:8])[0]
             self._assert(srs_id == geom_srs_id, 33,
                          ('table %s has geometries with SRID %d, ' +
                           'whereas only %d is expected') %
                          (table_name, geom_srs_id, srs_id))
+
+            self._assert(not (empty_flag and env_ind != 0), 152,
+                         "Invalid empty geometry")
+
             if env_ind == 0:
                 coord_dim = 0
             elif env_ind == 1:
@@ -439,12 +605,9 @@ class GPKGChecker(object):
             self._assert(len(blob) >= header_len, 19, 'Invalid geometry')
             wkb_endianness = blob_ar[header_len]
             wkb_big_endian = (wkb_endianness == 0)
-            if wkb_big_endian:
-                wkb_geom_type = struct.unpack(
-                    '>I' * 1, blob[header_len + 1:header_len + 5])[0]
-            else:
-                wkb_geom_type = struct.unpack(
-                    '<I' * 1, blob[header_len + 1:header_len + 5])[0]
+            wkb_endian_prefix = '>' if wkb_big_endian else '<'
+            wkb_geom_type = struct.unpack(
+                (wkb_endian_prefix + 'I') * 1, blob[header_len + 1:header_len + 5])[0]
             self._assert(wkb_geom_type >= 0 and
                          (wkb_geom_type % 1000) < len(wkb_geometries),
                          19, 'Invalid WKB geometry type')
@@ -467,6 +630,14 @@ class GPKGChecker(object):
 
             found_geom_types.add(wkb_geometries[wkb_geom_type % 1000])
 
+            if has_gdal:
+                geom = ogr.CreateGeometryFromWkb(blob[header_len:])
+                self._assert(geom is not None, 19, 'Invalid geometry')
+
+                self._assert((geom.IsEmpty() and empty_flag) or
+                             (not geom.IsEmpty() and not empty_flag), 152,
+                             'Inconsistent empty_flag vs geometry content')
+
         if geometry_type_name in ('POINT', 'LINESTRING', 'POLYGON',
                                   'MULTIPOINT', 'MULTILINESTRING',
                                   'MULTIPOLYGON'):
@@ -613,6 +784,52 @@ class GPKGChecker(object):
                           "gpkg_geometry_columns which isn't found in " +
                           "gpkg_spatial_ref_sys") % (table_name, srs_id))
 
+        c.execute("SELECT a.table_name, a.srs_id, b.srs_id FROM " +
+                  "gpkg_geometry_columns a, gpkg_contents b " +
+                  "WHERE a.table_name = b.table_name AND a.srs_id != b.srs_id")
+        rows = c.fetchall()
+        for (table_name, a_srs_id, b_srs_id) in rows:
+            self._assert(False, 146,
+                         "Table %s is declared with srs_id %d in "
+                         "gpkg_geometry_columns and %d in gpkg_contents" %
+                         (table_name, a_srs_id, b_srs_id))
+
+    def _check_attribute_user_table(self, c, table_name):
+        self._log('Checking attributes table ' + table_name)
+
+        c.execute('PRAGMA table_info(%s)' % _esc_id(table_name))
+        cols = c.fetchall()
+        count_pkid = 0
+        for (_, name, typ, _, _, pk) in cols:
+            if pk == 1:
+                count_pkid += 1
+                self._assert(typ == 'INTEGER', 119,
+                             ('table %s has a PRIMARY KEY of type %s ' +
+                              'instead of INTEGER') % (table_name, typ))
+
+            else:
+                self._assert(_is_valid_data_type(typ), 5,
+                             'table %s has column %s of unexpected type %s'
+                             % (table_name, name, typ))
+
+        c.execute("SELECT 1 FROM sqlite_master WHERE "
+                  "type = 'table' AND name = ?", (table_name,))
+        if c.fetchone():
+            self._assert(count_pkid == 1, 119,
+                         'table %s has no INTEGER PRIMARY KEY' % table_name)
+
+            self._check_user_table_content(c, table_name, cols)
+
+        else:
+            self._assert(len(cols) > 0 and cols[0][2] == 'INTEGER',
+                         151, 'view %s has no INTEGER first column' % table_name)
+
+            c.execute("SELECT COUNT(*) - COUNT(DISTINCT %s) FROM %s" %
+                      (_esc_id(cols[0][1]), _esc_id(table_name)))
+            self._assert(c.fetchone()[0] == 0, 151,
+                         'First column of view %s should contain '
+                         'unique values' % table_name)
+
     def _check_attributes(self, c):
 
         self._log('Checking attributes')
@@ -622,25 +839,7 @@ class GPKGChecker(object):
         if not rows:
             self._log('... No attributes table')
         for (table_name,) in rows:
-            self._log('Checking attributes table ' + table_name)
-
-            c.execute('PRAGMA table_info(%s)' % _esc_id(table_name))
-            cols = c.fetchall()
-            count_pkid = 0
-            for (_, name, typ, _, _, pk) in cols:
-                if pk == 1:
-                    count_pkid += 1
-                    self._assert(typ == 'INTEGER', 119,
-                                 ('table %s has a PRIMARY KEY of type %s ' +
-                                  'instead of INTEGER') % (table_name, typ))
-
-                else:
-                    self._assert(_is_valid_data_type(typ), 5,
-                                 'table %s has column %s of unexpected type %s'
-                                 % (table_name, name, typ))
-
-            self._assert(count_pkid == 1, 119,
-                         'table %s has no INTEGER PRIMARY KEY' % table_name)
+            self._check_attribute_user_table(c, table_name)
 
     def _check_tile_user_table(self, c, table_name, data_type):
 
@@ -1116,11 +1315,7 @@ class GPKGChecker(object):
                                  'Tile for %s should be PNG' % table_name)
                     if has_gdal:
                         tmp_file = '/vsimem/temp_validate_gpkg.tif'
-                        try:
-                            blob = bytes(blob)
-                        except:
-                            blob = str(blob)
-                        gdal.FileFromMemBuffer(tmp_file, blob)
+                        gdal.FileFromMemBuffer(tmp_file, bytes(blob))
                         ds = gdal.Open(tmp_file)
                         try:
                             self._assert(ds is not None, 'gpkg_2d_gridded_coverage#13',
@@ -1145,11 +1340,7 @@ class GPKGChecker(object):
                                  'Tile for %s should be TIFF' % table_name)
                     if has_gdal:
                         tmp_file = '/vsimem/temp_validate_gpkg.tif'
-                        try:
-                            blob = bytes(blob)
-                        except:
-                            blob = str(blob)
-                        gdal.FileFromMemBuffer(tmp_file, blob)
+                        gdal.FileFromMemBuffer(tmp_file, bytes(blob))
                         ds = gdal.Open(tmp_file)
                         try:
                             self._assert(ds is not None, 'gpkg_2d_gridded_coverage#15',
@@ -1186,6 +1377,17 @@ class GPKGChecker(object):
                             self._log('GDAL not available. '
                                       'Req gpkg_2d_gridded_coverage#15 to gpkg_2d_gridded_coverage#19 not tested')
 
+    def _check_column_exists(self, c, table_name, col_name):
+
+        try:
+            # 'SELECT "col_name" FROM table_name' doesn't work because
+            # it will be interpreted as a literal if the columm does not exist
+            c.execute('SELECT 1 FROM %s t ORDER BY t.%s LIMIT 0' %
+                      (_esc_id(table_name), _esc_id(col_name)))
+            return True
+        except sqlite3.OperationalError:
+            return False
+
     def _check_gpkg_extensions(self, c):
 
         self._log('Checking gpkg_extensions')
@@ -1219,15 +1421,10 @@ class GPKGChecker(object):
             #    'gpkg_extensions, but not in gpkg_contents') % table_name)
 
             if column_name is not None:
-                try:
-                    c.execute('SELECT %s FROM %s' %
-                              (_esc_id(column_name), _esc_id(table_name)))
-                    c.fetchone()
-                except:
-                    self._assert(False, 61,
-                                 ("Column %s of table %s mentioned in " +
-                                  "gpkg_extensions doesn't exist") %
-                                 (column_name, table_name))
+                self._assert(self._check_column_exists(c, table_name, column_name), 61,
+                             ("Column %s of table %s mentioned in " +
+                              "gpkg_extensions doesn't exist") %
+                             (column_name, table_name))
 
         c.execute("SELECT extension_name FROM gpkg_extensions")
         rows = c.fetchall()
@@ -1237,6 +1434,7 @@ class GPKGChecker(object):
                             'gpkg_metadata',
                             'gpkg_schema',
                             'gpkg_crs_wkt',
+                            'gpkg_crs_wkt_1_1',
                             'gpkg_elevation_tiles',  # deprecated one
                             'gpkg_2d_gridded_coverage'
                             ]
@@ -1422,13 +1620,9 @@ class GPKGChecker(object):
             self._assert(column_name is not None, 98,
                          "row in gpkg_metadata_reference with null "
                          "column_name")
-            try:
-                c.execute("SELECT %s FROM %s" %
-                          (_esc_id(column_name), _esc_id(table_name)))
-            except:
-                self._assert(False, 98,
-                             "column %s of %s does not exist" %
-                             (column_name, table_name))
+            self._assert(self._check_column_exists(c, table_name, column_name), 98,
+                         "column %s of %s does not exist" %
+                         (column_name, table_name))
 
         c.execute("SELECT table_name FROM "
                   "gpkg_metadata_reference WHERE "
@@ -1450,7 +1644,7 @@ class GPKGChecker(object):
                          "row in gpkg_metadata_reference with null "
                          "row_id_value")
             c.execute("SELECT 1 FROM %s WHERE ROWID = ?" %
-                      _esc_id(column_name), (row_id_value, ))
+                      _esc_id(table_name), (row_id_value, ))
             self._assert(c.fetchone() is not None, 99,
                          "row %s of %s does not exist" %
                          (str(row_id_value), table_name))
@@ -1495,9 +1689,7 @@ class GPKGChecker(object):
 
     def _check_schema(self, c):
 
-        # Partial: doesn't check gpkg_data_column_constraints
-
-        self._log('Checking gpkg_schema (partial)')
+        self._log('Checking gpkg_schema')
 
         must_have_gpkg_schema = False
         c.execute("SELECT 1 FROM sqlite_master WHERE name = 'gpkg_extensions'")
@@ -1512,10 +1704,28 @@ class GPKGChecker(object):
                              "Wrong scope for gpkg_schema in "
                              "gpkg_extensions")
 
+                self._assert(c.fetchone() is not None, 141,
+                             "There should be exactly 2 rows with " +
+                             "extension_name = " +
+                             "'gpkg_schema' in gpkg_extensions")
+                self._assert(c.fetchone() is None, 141,
+                             "There should be exactly 2 rows with " +
+                             "extension_name = " +
+                             "'gpkg_schema' in gpkg_extensions")
+
+                c.execute("SELECT 1 FROM gpkg_extensions WHERE "
+                          "extension_name = 'gpkg_schema' AND "
+                          "column_name IS NOT NULL")
+                row = c.fetchone()
+                if row is not None:
+                    self._assert(False, 141,
+                                 "gpkg_extensions contains row(s) with " +
+                                 "gpkg_schema and a not-NULL column_name")
+
         c.execute("SELECT 1 FROM sqlite_master WHERE name = 'gpkg_data_columns'")
         if c.fetchone() is None:
             if must_have_gpkg_schema:
-                self._log("gpkg_data_columns table missing. Not forbidden by requirements, but odd")
+                self._assert(False, 141, "gpkg_data_columns table missing.")
             else:
                 self._log('... No schema')
             return
@@ -1531,7 +1741,7 @@ class GPKGChecker(object):
             (5, 'mime_type', 'TEXT', 0, None, 0),
             (6, 'constraint_name', 'TEXT', 0, None, 0)
         ]
-        self._check_structure(columns, expected_columns, 107,
+        self._check_structure(columns, expected_columns, 103,
                               'gpkg_data_columns')
 
         c.execute("SELECT table_name, column_name FROM gpkg_data_columns")
@@ -1539,20 +1749,116 @@ class GPKGChecker(object):
         for (table_name, column_name) in rows:
             c.execute("SELECT 1 FROM gpkg_contents WHERE table_name = ?",
                       (table_name,))
-            self._assert(c.fetchone(), 104,
-                         ("table_name = %s " % table_name +
-                          "in gpkg_data_columns refer to non-existing " +
-                          "table/view in gpkg_contents"))
-
-            try:
-                c.execute("SELECT %s FROM %s" % (_esc_id(column_name),
-                                                 _esc_id(table_name)))
-            except sqlite3.OperationalError:
-                self._assert(False, 105,
-                             ("table_name = %s, " % table_name +
-                              "column_name = %s " % column_name +
+            if c.fetchone() is None:
+                c.execute("SELECT 1 FROM gpkg_extensions WHERE table_name = ?",
+                          (table_name,))
+                self._assert(c.fetchone(), 104,
+                             ("table_name = %s " % table_name +
                               "in gpkg_data_columns refer to non-existing " +
-                              "column"))
+                              "table/view in gpkg_contents or gpkg_extensions"))
+
+            self._assert(self._check_column_exists(c, table_name, column_name), 105,
+                         ("table_name = %s, " % table_name +
+                          "column_name = %s " % column_name +
+                          "in gpkg_data_columns refer to non-existing " +
+                          "column"))
+
+        c.execute("SELECT 1 FROM sqlite_master WHERE name = 'gpkg_data_column_constraints'")
+        if c.fetchone() is None:
+            self._assert(False, 141, "gpkg_data_column_constraints table missing.")
+
+        c.execute("PRAGMA table_info(gpkg_data_column_constraints)")
+        columns = c.fetchall()
+        expected_columns = [
+            (0, 'constraint_name', 'TEXT', 1, None, 0),
+            (1, 'constraint_type', 'TEXT', 1, None, 0),
+            (2, 'value', 'TEXT', 0, None, 0),
+            (3, 'min', 'NUMERIC', 0, None, 0),
+            (4, 'min_is_inclusive', 'BOOLEAN', 0, None, 0),
+            (5, 'max', 'NUMERIC', 0, None, 0),
+            (6, 'max_is_inclusive', 'BOOLEAN', 0, None, 0),
+            (7, 'description', 'TEXT', 0, None, 0),
+        ]
+        self._check_structure(columns, expected_columns, 107,
+                              'gpkg_data_column_constraints')
+
+        c.execute("SELECT DISTINCT constraint_type FROM " +
+                  "gpkg_data_column_constraints WHERE constraint_type " +
+                  "NOT IN ('range', 'enum', 'glob')")
+        if c.fetchone() is not None:
+            self._assert(False, 108,
+                         "gpkg_data_column_constraints.constraint_type " +
+                         "contains value other than range, enum, glob")
+
+        c.execute("SELECT 1 FROM (SELECT COUNT(constraint_name) AS c FROM " +
+                  "gpkg_data_column_constraints WHERE constraint_type " +
+                  "IN ('range', 'glob') GROUP BY constraint_name) u " +
+                  "WHERE u.c != 1")
+        if c.fetchone() is not None:
+            self._assert(False, 109,
+                         "gpkg_data_column_constraints contains non unique " +
+                         "constraint_name for constraints of type range/glob")
+
+        c.execute("SELECT 1 FROM gpkg_data_column_constraints WHERE " +
+                  "constraint_type = 'range' AND value IS NOT NULL")
+        if c.fetchone() is not None:
+            self._assert(False, 110,
+                         "gpkg_data_column_constraints contains constraint " +
+                         "of type range whose 'value' column is not null")
+
+        c.execute("SELECT 1 FROM gpkg_data_column_constraints WHERE " +
+                  "constraint_type = 'range' AND min IS NULL")
+        if c.fetchone() is not None:
+            self._assert(False, 111,
+                         "gpkg_data_column_constraints contains constraint " +
+                         "of type range whose min value is NULL")
+
+        c.execute("SELECT 1 FROM gpkg_data_column_constraints WHERE " +
+                  "constraint_type = 'range' AND max IS NULL")
+        if c.fetchone() is not None:
+            self._assert(False, 111,
+                         "gpkg_data_column_constraints contains constraint " +
+                         "of type range whose max value is NULL")
+
+        c.execute("SELECT 1 FROM gpkg_data_column_constraints WHERE " +
+                  "constraint_type = 'range' AND min >= max")
+        if c.fetchone() is not None:
+            self._assert(False, 111,
+                         "gpkg_data_column_constraints contains constraint " +
+                         "of type range whose min value is not less than max")
+
+        c.execute("SELECT 1 FROM gpkg_data_column_constraints WHERE " +
+                  "constraint_type = 'range' AND min_is_inclusive NOT IN (0,1)")
+        if c.fetchone() is not None:
+            self._assert(False, 112,
+                         "gpkg_data_column_constraints contains constraint " +
+                         "of type range whose min_is_inclusive value is " +
+                         "not 0 or 1")
+
+        c.execute("SELECT 1 FROM gpkg_data_column_constraints WHERE " +
+                  "constraint_type = 'range' AND max_is_inclusive NOT IN (0,1)")
+        if c.fetchone() is not None:
+            self._assert(False, 112,
+                         "gpkg_data_column_constraints contains constraint " +
+                         "of type range whose max_is_inclusive value is " +
+                         "not 0 or 1")
+
+        for col_name in ('min', 'min_is_inclusive', 'max', 'max_is_inclusive'):
+            c.execute("SELECT 1 FROM gpkg_data_column_constraints WHERE " +
+                      "constraint_type IN ('enum', 'glob') AND " +
+                      col_name + " IS NOT NULL")
+            if c.fetchone() is not None:
+                self._assert(False, 113,
+                             "gpkg_data_column_constraints contains constraint " +
+                             "of type enum or glob whose " + col_name +
+                             " column is not NULL")
+
+        c.execute("SELECT 1 FROM gpkg_data_column_constraints WHERE " +
+                  "constraint_type = 'enum' AND value IS NULL")
+        if c.fetchone() is not None:
+            self._assert(False, 114,
+                         "gpkg_data_column_constraints contains constraint " +
+                         "of type enum whose value column is NULL")
 
     def check(self):
         self._assert(os.path.exists(self.filename), None,
@@ -1632,52 +1938,85 @@ class GPKGChecker(object):
             conn.close()
 
 
-def check(filename, abort_at_first_error=True, verbose=False):
+def check(filename,
+          abort_at_first_error=True,
+          verbose=None,  # deprecated
+          extra_checks=False,
+          log_msg=False,
+          warning_msg=True,
+          warning_as_error=False):
+
+    if verbose is not None:
+        if verbose:
+            log_msg = True
+            warning_msg = True
+        else:
+            log_msg = False
+            warning_msg = False
 
     checker = GPKGChecker(filename,
                           abort_at_first_error=abort_at_first_error,
-                          verbose=verbose)
+                          extra_checks=extra_checks,
+                          log_msg=log_msg,
+                          warning_msg=warning_msg,
+                          warning_as_error=warning_as_error)
     checker.check()
     return checker.errors
 
 
 def Usage():
-    print('validate_gpkg.py [[-v]|[-q]] [-k] my.gpkg')
+    print('validate_gpkg.py [[-v]|[-q]] [-k] [--extra] [--warning-as-error] my.gpkg')
     print('')
+    print('-v: verbose mode')
     print('-q: quiet mode')
     print('-k: (try to) keep going when error is encountered')
-    sys.exit(1)
+    print('--extra: run extra checks, potentially going beyond strict requirements of specification')
+    print('--warning-as-error: turn warnings as errors')
+    return 1
 
 
-if __name__ == '__main__':
-
+def main(argv):
     filename = None
-    verbose = False
+    log_msg = False
+    warning_msg = True
     abort_at_first_error = True
-    if len(sys.argv) == 1:
-        Usage()
-    for arg in sys.argv[1:]:
+    warning_as_error = False
+    extra_checks = False
+    if len(argv) == 1:
+        return Usage()
+    for arg in argv[1:]:
         if arg == '-k':
             abort_at_first_error = False
         elif arg == '-q':
-            verbose = False
+            warning_msg = False
         elif arg == '-v':
-            verbose = True
+            log_msg = True
+        elif arg == '--extra':
+            extra_checks = True
+        elif arg == '--warning-as-error':
+            warning_as_error = True
         elif arg[0] == '-':
-            Usage()
+            return Usage()
         else:
             filename = arg
     if filename is None:
-        Usage()
+        return Usage()
     ret = check(filename, abort_at_first_error=abort_at_first_error,
-                verbose=verbose)
+                extra_checks=extra_checks,
+                log_msg=log_msg, warning_msg=warning_msg,
+                warning_as_error=warning_as_error)
     if not abort_at_first_error:
         if not ret:
-            sys.exit(0)
+            return 0
         else:
             for (req, msg) in ret:
                 if req:
                     print('Req %d: %s' % (req, msg))
                 else:
                     print(msg)
-            sys.exit(1)
+            return 1
+    return 0
+
+
+if __name__ == '__main__':
+    sys.exit(main(sys.argv))
